<?php

declare(strict_types=1);

namespace Drupal\Tests\data_pipelines\Kernel;

use Drupal\data_pipelines\Entity\Dataset;

/**
 * Defines a test for DatasetPipelineBase.
 *
 * @group data_pipelines
 * @covers \Drupal\data_pipelines\DatasetPipelineBase
 * @covers \Drupal\data_pipelines\Entity\Dataset::validate
 * @covers \Drupal\data_pipelines\Plugin\DataType\DatasetData
 * @covers \Drupal\data_pipelines\Plugin\Validation\Constraint\ItemCountValidator
 */
class DatasetPipelineBaseTest extends DatasetKernelTestBase {

  /**
   * Tests validation on a valid CSV.
   */
  public function testValidationValidCsv() {
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1.csv');
    $dataset = Dataset::create([
      'source' => 'csv:file',
      'destinations' => [$this->createTestMemoryDestination()],
      'name' => $this->randomMachineName(),
      'machine_name' => mb_strtolower($this->randomMachineName()),
      'pipeline' => 'test_pipeline_1',
      'csv_file' => $file,
    ]);
    $this->assertTrue($dataset->hasValidation());
    $this->assertEmpty($dataset->validate());
  }

  /**
   * Tests has validation method.
   *
   * @covers \Drupal\data_pipelines\Entity\Dataset::hasValidation
   * @covers \Drupal\data_pipelines\DatasetPipelineBase::hasValidation
   */
  public function testHasValidation(): void {
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1.csv');
    $dataset = Dataset::create([
      'source' => 'csv:file',
      'destinations' => [$this->createTestMemoryDestination()],
      'name' => $this->randomMachineName(),
      'machine_name' => mb_strtolower($this->randomMachineName()),
      'pipeline' => 'pipeline_with_no_validation',
      'csv_file' => $file,
    ]);
    $this->assertFalse($dataset->hasValidation());
  }

  /**
   * Tests validation on a valid JSON file.
   */
  public function testValidationValidJson() {
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1.json');
    $dataset = Dataset::create([
      'source' => 'json:file',
      'destinations' => [$this->createTestMemoryDestination()],
      'name' => $this->randomMachineName(),
      'machine_name' => mb_strtolower($this->randomMachineName()),
      'pipeline' => 'test_pipeline_1',
      'json_file' => $file,
    ]);
    $this->assertEmpty($dataset->validate());
  }

  /**
   * Tests validation on an invalid CSV.
   */
  public function testValidationInvalidCsv() {
    // @todo remove this once Drupal 9 is not supported anymore.
    $d10 = version_compare(\Drupal::VERSION, '10', '>');
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1-invalid.csv');
    $dataset = Dataset::create([
      'source' => 'csv:file',
      'destinations' => [$this->createTestMemoryDestination()],
      'name' => $this->randomMachineName(),
      'machine_name' => mb_strtolower($this->randomMachineName()),
      'pipeline' => 'test_pipeline_1',
      'csv_file' => $file,
    ]);
    $result = $dataset->validate();
    // Symfony 4 and Symfony 5 has different violation count.
    // @todo update this once Drupal 9 is not supported anymore.
    $this->assertCount($d10 ? 3 : 2, $result);
    $violation = $result->get(0);
    $this->assertEquals('Firstname should be at least 3 characters', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('jo', $violation->getInvalidValue());
    $violation = $result->get(1);
    $this->assertEquals('Firstname is required', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('', $violation->getInvalidValue());
    // Symfony 4 and Symfony 5 has different violation count.
    // @todo update this once Drupal 9 is not supported anymore.
    if ($d10) {
      $violation = $result->get(2);
      $this->assertEquals('Firstname should be at least 3 characters', $violation->getMessage());
      $this->assertEquals('firstname', $violation->getPropertyPath());
      $this->assertEquals('', $violation->getInvalidValue());
    }
    // If the entity is syncing, we don't validate the dataset.
    $dataset->setSyncing(TRUE);
    $result = $dataset->validate();
    $this->assertCount(0, $result);
  }

  /**
   * Tests validation on an invalid JSON file.
   */
  public function testValidationInvalidJson() {
    // @todo remove this once Drupal 9 is not supported anymore.
    $d10 = version_compare(\Drupal::VERSION, '10', '>');
    $file = $this->getTestFile(dirname(__DIR__, 1) . '/fixtures/test-pipeline-1-invalid.json');
    $dataset = Dataset::create([
      'source' => 'json:file',
      'destinations' => [$this->createTestMemoryDestination()],
      'pipeline' => 'test_pipeline_1',
      'name' => $this->randomMachineName(),
      'machine_name' => mb_strtolower($this->randomMachineName()),
      'json_file' => $file,
    ]);
    $result = $dataset->validate();
    // Symfony 4 and Symfony 5 has different violation count.
    // @todo update this once Drupal 9 is not supported anymore.
    $this->assertCount($d10 ? 4 : 3, $result);
    $violation = $result->get(0);
    $this->assertEquals('Firstname should be at least 3 characters', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('jo', $violation->getInvalidValue());
    $violation = $result->get(1);
    $this->assertEquals('Firstname is required', $violation->getMessage());
    $this->assertEquals('firstname', $violation->getPropertyPath());
    $this->assertEquals('', $violation->getInvalidValue());
    // Symfony 4 and Symfony 5 has different violation count.
    // @todo update this once Drupal 9 is not supported anymore.
    if ($d10) {
      $violation = $result->get(2);
      $this->assertEquals('Firstname should be at least 3 characters', $violation->getMessage());
      $this->assertEquals('firstname', $violation->getPropertyPath());
      $this->assertEquals('', $violation->getInvalidValue());
    }
    // Symfony 4 and Symfony 5 has different violation count.
    // @todo update this once Drupal 9 is not supported anymore.
    $violation = $result->get($d10 ? 3 : 2);
    $this->assertEquals('All records should have 3 values', $violation->getMessage());
    $this->assertEquals('', $violation->getPropertyPath());
    $this->assertEquals([
      'firstname' => 'missing',
      'lastname' => 'a value',
    ], $violation->getInvalidValue());
  }

}
